//
//  StoryBlock.swift
//  fw_flutter_sdk
//
//  Created by linjie jiang on 2022/8/10.
//

import FireworkVideo
import UIKit

@objc
public enum StoryBlockSourceType: Int {
    case discover, channel, playlist, dynamicContent, hashtagPlaylist
}

public protocol StoryBlockDelegate: AnyObject {
    func storyBlockDidLoadFeed(_ view: StoryBlock)
    func storyBlock(_ view: StoryBlock, didFailToLoadFeed error: StoryBlockError)
}

@objc
public class StoryBlock: UIView, StoryBlockViewControllerDelegate {
    @objc public var sourceType: StoryBlockSourceType = .discover
    @objc public var channel: String = ""
    @objc public var playlist: String = ""
    @objc public var dynamicContentParameters: [String: [String]] = [:]
    @objc public var hashtagFilterExpression: String = ""
    @objc public var enablePictureInPicture: Bool = false
    @objc public var adConfiguration: AdConfiguration?

    public weak var delegate: StoryBlockDelegate?

    private var storyBlockVC: StoryBlockViewController?

    private var source: StoryBlockContentSource {
        switch sourceType {
        case .discover:
            return .discover
        case .channel:
            return .channel(channelID: channel)
        case .playlist:
            return .channelPlaylist(channelID: channel, playlistID: playlist)
        case .dynamicContent:
            return .dynamicContent(channelID: channel, parameters: dynamicContentParameters)
        case .hashtagPlaylist:
            return .hashtagPlaylist(channelID: channel, filterExpression: hashtagFilterExpression)
        }
    }

    public override init(frame: CGRect) {
        super.init(frame: frame)
        self.clipsToBounds = true
    }

    required init?(coder: NSCoder) {
        super.init(coder: coder)
        self.clipsToBounds = true
    }

    public override func layoutSubviews() {
        super.layoutSubviews()

        embed()
    }

    private func embed() {
        guard let parentVC = parentViewController else {
            return
        }
        guard self.storyBlockVC == nil else {
            return
        }

        var resultStoryBlockVC: StoryBlockViewController?
        if let fireworkVideoAdConfiguration = Convert.getFireworkVideoAdConfiguration(adConfiguration) {
            resultStoryBlockVC = StoryBlockViewController(source: source, adConfiguration: fireworkVideoAdConfiguration)
        } else {
            resultStoryBlockVC = StoryBlockViewController(source: source)
        }

        guard let storyBlockVC = resultStoryBlockVC else {
            return
        }

        if self.enablePictureInPicture {
            storyBlockVC.isPictureInPictureEnabled = true
        }

        storyBlockVC.delegate = self
        self.storyBlockVC = storyBlockVC
        parentVC.attachChild(storyBlockVC, to: self)
    }

    public override func removeFromSuperview() {
        super.removeFromSuperview()

        guard let storyBlockVC = self.storyBlockVC else {
            return
        }

        storyBlockVC.detachFromParent()
        self.storyBlockVC = nil
    }

    public func play() {
        guard let storyBlockVC = self.storyBlockVC else {
            return
        }
        storyBlockVC.play()
    }

    public func pause() {
        guard let storyBlockVC = self.storyBlockVC else {
            return
        }
        storyBlockVC.pause()
    }

    public func storyBlockDidLoadFeed(_ viewController: StoryBlockViewController) {
        guard let delegate = self.delegate else {
            return
        }

        delegate.storyBlockDidLoadFeed(self)
    }

    public func storyBlock(
        _ viewController: StoryBlockViewController, didFailToLoadFeed error: StoryBlockError
    ) {
        guard let delegate = self.delegate else {
            return
        }

        delegate.storyBlock(self, didFailToLoadFeed: error)
    }
}
