//
//  FireworkSDKPlugin.swift
//  fw_flutter_sdk
//
//  Created by Jeff Zheng on 2022/2/21.
//

import FireworkVideo
import Flutter
import Foundation

class FireworkSDKPlugin: NSObject, FlutterPlugin, FireworkVideoSDKDelegate {
    static var shareBaseURL: String?
    static var adBadgeConfiguration: AdBadgeConfiguration?
    static var videoLaunchBehavior: VideoLaunchBehavior?

    var customCTAClickEnabled: Bool = false

    /// Decide whether to send video playback event to Dart.
    var enableVideoPlayBackEvent: Bool = false

    var channel: FlutterMethodChannel! = nil

    static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(
            name: ChannelName.fireworkSDKModule.rawValue, binaryMessenger: registrar.messenger())
        let instance = FireworkSDKPlugin()
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    override init() {
        super.init()
        _ = AppLanguageManager.shared
        NotificationCenter.default.addObserver(
            self, selector: #selector(Self.onAppLanguageChanged),
            name: AppLanguageManager.NotificationName.AppLanguageChanged, object: nil)
    }

    func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case FireworkSDKPluginMethodName.`init`.rawValue:
            initializeSDK(call.arguments as? [String: AnyObject])
            result(nil)
        case FireworkSDKPluginMethodName.openVideoPlayer.rawValue:
            openVideoPlayer(call.arguments as? [String: AnyObject])
            result(nil)
        case FireworkSDKPluginMethodName.setCustomCTAClickEnabled.rawValue:
            setCustomCTAClickEnabled(call.arguments as? Bool)
            result(nil)
        case FireworkSDKPluginMethodName.setVideoPlaybackEventEnabled.rawValue:
            setVideoPlaybackEventEnabled(call.arguments as? Bool)
            result(nil)
        case FireworkSDKPluginMethodName.setShareBaseURL.rawValue:
            setShareBaseURL(call.arguments as? String)
            result(nil)
        case FireworkSDKPluginMethodName.setAdBadgeConfiguration.rawValue:
            setAdBadgeConfiguration(call.arguments as? [String: AnyObject])
            result(nil)
        case FireworkSDKPluginMethodName.trackPurchase.rawValue:
            trackPurchase(call.arguments as? [String: AnyObject])
            result(nil)
        case FireworkSDKPluginMethodName.changeAppLanguage.rawValue:
            changeAppLanguage(call.arguments as? String, result: result)
        default:
            result(FlutterMethodNotImplemented)
        }
    }

    private func initializeSDK(_ options: [String: AnyObject]?) {
        let sdkInitOptions = Convert.sdkInitOptions(options)

        FireworkVideoSDK.initializeSDK(delegate: self, userID: sdkInitOptions?.userId)
        FireworkVideoSDK.ctaDelegate = self
        FireworkVideoSDK.eventTracking.videoPlaybackDelegate = self
        FireworkVideoSDK.eventTracking.feedDelegate = self

        Self.videoLaunchBehavior = sdkInitOptions?.videoLaunchBehavior
    }

    private func openVideoPlayer(_ config: [String: AnyObject]?) {
        guard let presentedVC = UIViewController.presentedViewController() else {
            return
        }

        guard let urlString = config?["url"] as? String, let url = URL.init(string: urlString) else {
            return
        }

        let vpcConfig = Convert.videoPlayerContentConfiguration(
            config?["config"] as? [String: AnyObject])

        var feedContentConfig = VideoFeedContentConfiguration()
        if let vpcConfig = vpcConfig {
            feedContentConfig.playerView = vpcConfig
        }
        if let adBadge = Self.convertToFWAdBadgeConfiguration(Self.adBadgeConfiguration) {
            feedContentConfig.adBadge = adBadge
        }

        VideoFeedViewController.openVideoPlayer(with: url, feedContentConfig, presentedVC) { _ in

        }
    }

    private func setCustomCTAClickEnabled(_ enabled: Bool?) {
        if let enabled = enabled {
            customCTAClickEnabled = enabled
        }
    }

    private func setVideoPlaybackEventEnabled(_ enabled: Bool?) {
        if let enabled = enabled {
            enableVideoPlayBackEvent = enabled
        }
    }

    func setShareBaseURL(_ url: String?) {
        Self.shareBaseURL = url
    }

    func setAdBadgeConfiguration(_ config: [String: AnyObject]?) {
        guard let adBadgeConfiguration =
                JsonHelper.fromDictionary(config, type: AdBadgeConfiguration.self) else {
            return
        }

        Self.adBadgeConfiguration = adBadgeConfiguration
    }

    func trackPurchase(_ config: [String: AnyObject]?) {
        guard
            let trackPurchaseParameters = JsonHelper.fromDictionary(
                config, type: TrackPurchaseParameters.self)
        else {
            return
        }

        FireworkVideoSDK.trackPurchase(
            orderID: trackPurchaseParameters.orderId,
            value: trackPurchaseParameters.value,
            currencyCode: trackPurchaseParameters.currencyCode,
            countryCode: trackPurchaseParameters.countryCode,
            trackPurchaseParameters.additionalInfo
        )
    }

    func changeAppLanguage(_ language: String?, result: @escaping FlutterResult) {
        let changeAppLanguageResult = AppLanguageManager.shared.changeAppLanguage(language)
        result(changeAppLanguageResult)
    }

    // MARK: - FireworkVideoSDKDelegate
    func fireworkVideoDidLoadSuccessfully() {
        channel.invokeMethod(FWEventName.sdkInit.rawValue, arguments: [:])
    }

    func fireworkVideoDidLoadWith(error: FireworkVideoSDKError) {
        channel.invokeMethod(
            FWEventName.sdkInit.rawValue,
            arguments: [
                "error": [
                    "name": error.jsErrorName, "reason": error.recoverySuggestion ?? "default"
                ]
            ])
    }

    @objc func onAppLanguageChanged() {
        channel.invokeMethod(
            FWEventName.nativeAppLanguageUpdated.rawValue,
            arguments: [
                "appLanguage": AppLanguageManager.shared.appLanguage ?? ""
            ])
    }
}

extension FireworkSDKPlugin {
    static func convertToFWAdBadgeConfiguration(_ adBadgeConfiguration: AdBadgeConfiguration?)
        -> FireworkVideo.AdBadgeConfiguration? {
        guard let adBadgeConfiguration = adBadgeConfiguration else {
            return nil
        }

        var fwAdBadgeConfiguration = FireworkVideo.AdBadgeConfiguration()
        if let textColor = adBadgeConfiguration.textColor {
            fwAdBadgeConfiguration.textColor = textColor.uicolor()
        }
        if let backgroundColor = adBadgeConfiguration.backgroundColor {
            fwAdBadgeConfiguration.backgroundColor = backgroundColor.uicolor()
        }
        switch adBadgeConfiguration.badgeTextType {
        case .sponsored:
            fwAdBadgeConfiguration.badgeText =
                FireworkVideo.AdBadgeConfiguration.BadgeText.sponsored
        default:
            fwAdBadgeConfiguration.badgeText = FireworkVideo.AdBadgeConfiguration.BadgeText.ad
        }

        return fwAdBadgeConfiguration
    }
}
