import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

import '../../constants/fw_event_name.dart';
import '../../firework_sdk.dart';
import '../../fw_event_bus.dart';
import '../../models/ad_badge_configuration.dart';
import '../../models/ad_configuration.dart';
import '../../models/fw_error.dart';
import '../../models/story_block_configuration.dart';
import '../../models/story_block_source.dart';
import '../../utils/fw_global_state.dart';
import 'story_block_controller.dart';
import 'story_block_platform_handler.dart';
import 'story_block_platform_view.dart';

typedef StoryBlockLoadFinishedCallback = void Function(FWError? error);
typedef StoryBlockCreatedCallback = void Function(
    StoryBlockController controller);

// The StoryBlock widget is only supported on iOS.
class StoryBlock extends StatefulWidget {
  /// The source of the story block.
  final StoryBlockSource source;

  /// The channel id of the story block.
  ///
  /// Required when the source is set as `channel` or `playlist` or `dynamicContent`.
  final String? channel;

  /// The playlist id of the story block.
  ///
  /// Required when the source is set as `playlist`.
  final String? playlist;

  /// The dynamic content parameters of the story block.
  ///
  /// Required when the source is set as `dynamicContent`.
  final Map<String, List<String>>? dynamicContentParameters;

  /// Hashtag filter expression is an s-expression used to provide feeds filtered by hashtags with specified criteria.
  /// Queries are specified with boolean predicates on what hashtags are there on the video.
  /// For instance, (and sport food) (or sport food) (and sport (or food comedy)) sport are all valid expressions.
  /// Non-UTF-8 characters are not allowed. If using boolean predicates, the expression needs to be wrapped with parenthesis.
  final String? hashtagFilterExpression;

  /// Specifies if Picture in Picture is enabled. Only supported on iOS.
  final bool? enablePictureInPicture;

  /// Ad configuration of the feed. Only supported on iOS.
  final AdConfiguration? adConfiguration;

  /// The configuration of the story block.
  /// Only supported on Android.
  final StoryBlockConfiguration? storyBlockConfiguration;

  /// The width of the story block.
  final double? width;

  /// The height of the story block.
  final double? height;

  /// The callback of story block creation.
  final StoryBlockCreatedCallback? onStoryBlockCreated;

  /// The callback of the feed loading result.
  ///
  /// It means loading successfully when error equals to null.
  final StoryBlockLoadFinishedCallback? onStoryBlockLoadFinished;

  const StoryBlock({
    Key? key,
    required this.source,
    this.channel,
    this.playlist,
    this.dynamicContentParameters,
    this.hashtagFilterExpression,
    this.enablePictureInPicture,
    this.adConfiguration,
    this.storyBlockConfiguration,
    this.width,
    this.height,
    this.onStoryBlockCreated,
    this.onStoryBlockLoadFinished,
  }) : super(key: key);

  @override
  State<StoryBlock> createState() => _StoryBlockState();
}

class _StoryBlockState extends State<StoryBlock> {
  StoryBlockController? _controller;
  StreamSubscription? _eventSubscription;
  bool _sdkInitCalled = false;
  bool _isFullScreen = false;

  @override
  void initState() {
    super.initState();
    _eventSubscription = FWEventBus.getInstance().on().listen((event) {
      if (mounted) {
        if (event.eventName == FWEventName.adBadgeConfigurationUpdated) {
          setState(() {});
        } else if (event.eventName == FWEventName.shareBaseURLUpdated) {
          setState(() {});
        } else if (event.eventName == FWEventName.videoLaunchBehaviorUpdated) {
          setState(() {});
        } else if (event.eventName == FWEventName.appLanguageUpdated) {
          setState(() {});
        }
      }
    });

    _sdkInitCalled = FWGlobalState.getInstance().sdkInitCalled;
    if (!_sdkInitCalled) {
      WidgetsBinding.instance.addPostFrameCallback((_) {
        FWGlobalState.getInstance().sdkInitCalledFuture.then((__) {
          setState(() {
            _sdkInitCalled = true;
          });
        });
      });
    }
  }

  @override
  void dispose() {
    _eventSubscription?.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (!_sdkInitCalled) {
      return SizedBox(
        width: widget.width,
        height: widget.height,
      );
    }

    return WillPopScope(
        child: SizedBox(
          width: widget.width,
          height: widget.height,
          child: StoryBlockPlatformView(
            key: ValueKey(_generateKey()),
            source: widget.source,
            channel: widget.channel,
            playlist: widget.playlist,
            dynamicContentParameters: widget.dynamicContentParameters,
            hashtagFilterExpression: widget.hashtagFilterExpression,
            enablePictureInPicture: widget.enablePictureInPicture,
            adConfiguration: widget.adConfiguration,
            storyBlockConfiguration: widget.storyBlockConfiguration,
            onStoryBlockformViewCreated: _onStoryBlockPlatformViewCreated,
            onStoryBlockPlatformViewLoadFinished:
                widget.onStoryBlockLoadFinished,
            onStoryBlockPlatformViewFullScreenStateChanged: (isFullScreen) {
              setState(() {
                _isFullScreen = isFullScreen;
              });
            },
          ),
        ),
        onWillPop: () async {
          if (defaultTargetPlatform == TargetPlatform.android &&
              _isFullScreen) {
            FireworkSDK.getInstance().navigator.popNativeContainer();
            return false;
          }

          return true;
        });
  }

  String _generateKey() {
    final gShareBaseURL = FireworkSDK.getInstance().shareBaseURL ?? "";
    final appLanguage = FireworkSDK.getInstance().appLanguage ?? "";
    final videoLaunchBehavior =
        FireworkSDK.getInstance().videoLaunchBehavior?.name ?? "";
    final adBadgeConfiguration =
        FireworkSDK.getInstance().adBadgeConfiguration ??
            AdBadgeConfiguration();
    final adBadgeTextType = adBadgeConfiguration.badgeTextType?.name ?? "";
    final backgroundColorOfAdBadge = adBadgeConfiguration.backgroundColor ?? "";
    final textColorOfAdBadge = adBadgeConfiguration.textColor ?? "";
    final androidFontIsCustomOfAdBadge =
        adBadgeConfiguration.androidFontInfo?.isCustom ?? false;
    final androidFontTypefaceNameOfAdBadge =
        adBadgeConfiguration.androidFontInfo?.typefaceName ?? "";

    final source = widget.source.name;
    final channel = widget.channel ?? "";
    final playlist = widget.playlist ?? "";
    final dynamicContentParametersString =
        _generateDynamicContentParametersString();
    final hashtagFilterExpression = widget.hashtagFilterExpression ?? "";
    final enablePictureInPicture = widget.enablePictureInPicture ?? false;

    final videoCompleteAction =
        widget.storyBlockConfiguration?.videoCompleteAction?.name ?? "";
    final showShareButton =
        widget.storyBlockConfiguration?.showShareButton ?? false;
    final showPlaybackButton =
        widget.storyBlockConfiguration?.showPlaybackButton ?? false;
    final showBranding = widget.storyBlockConfiguration?.showBranding ?? false;
    final ctaDelay = widget.storyBlockConfiguration?.ctaDelay;
    final ctaDelayType = ctaDelay?.type.name ?? "";
    final ctaHighlightDelay = widget.storyBlockConfiguration?.ctaHighlightDelay;
    final ctaHighlightDelayType = ctaHighlightDelay?.type.name ?? "";
    final ctaHighlightDelayValue =
        ctaHighlightDelay?.value.toStringAsFixed(5) ?? "";
    final ctaDelayValue = ctaDelay?.value.toStringAsFixed(5) ?? "";
    final shareBaseURL = widget.storyBlockConfiguration?.shareBaseURL ?? "";
    final ctaWidth = widget.storyBlockConfiguration?.ctaWidth?.name ?? "";

    final requiresAds = widget.adConfiguration?.requiresAds ?? false;
    final adsFetchTimeout = widget.adConfiguration?.adsFetchTimeout ?? 10;
    final vastAttributesString = _generateVastAttributesString();

    String key = "_gShareBaseURL:$gShareBaseURL";
    if (defaultTargetPlatform == TargetPlatform.iOS) {
      key += "_appLanguage:$appLanguage";
    }
    key += "_videoLaunchBehavior:$videoLaunchBehavior";
    key += "_adBadgeTextType:$adBadgeTextType";
    key += "_backgroundColorOfAdBadge:$backgroundColorOfAdBadge";
    key += "_textColorOfAdBadge:$textColorOfAdBadge";
    if (defaultTargetPlatform == TargetPlatform.android) {
      key += "_androidFontIsCustomOfAdBadge:$androidFontIsCustomOfAdBadge";
      key +=
          "_androidFontTypefaceNameOfAdBadge:$androidFontTypefaceNameOfAdBadge";
    }
    key += "_source:$source";
    key += "_channel:$channel";
    key += "_playlist:$playlist";
    key += "_dynamicContentParameters:$dynamicContentParametersString";
    key += "hashtagFilterExpression:$hashtagFilterExpression";
    key += "_enablePictureInPicture:$enablePictureInPicture";

    if (defaultTargetPlatform == TargetPlatform.android) {
      key += "_videoCompleteAction:$videoCompleteAction";
      key += "_showShareButton:$showShareButton";
      key += "_showPlaybackButton:$showPlaybackButton";
      key += "_showBranding:$showBranding";
      key += "_ctaDelayType:$ctaDelayType";
      key += "_ctaDelayValue:$ctaDelayValue";
      key += "_ctaHighlightDelayType:$ctaHighlightDelayType";
      key += "_ctaHighlightDelayValue:$ctaHighlightDelayValue";
      key += "_shareBaseURL:$shareBaseURL";
      key += "_ctaWidth:$ctaWidth";
    }

    key += "_requiresAds:$requiresAds";
    key += "_adsFetchTimeout:$adsFetchTimeout";
    key += "_vastAttributesString:$vastAttributesString";
    return key;
  }

  String _generateDynamicContentParametersString() {
    final parameters = widget.dynamicContentParameters;
    if (parameters == null) {
      return "";
    }

    var resultString = "";
    final sortedKeyList = parameters.keys.toList()..sort();
    for (final key in sortedKeyList) {
      final value = parameters[key];
      if (value != null) {
        final valueString = value.join(",");
        if (resultString.isNotEmpty) {
          resultString += "_";
        }
        resultString += "$key:$valueString";
      }
    }

    return resultString;
  }

  String _generateVastAttributesString() {
    final vastAttributes = widget.adConfiguration?.vastAttributes;
    if (vastAttributes == null) {
      return "";
    }

    var resultString = "";
    for (var attribute in vastAttributes) {
      final name = attribute.name ?? "";
      final value = attribute.value ?? "";
      if (resultString.isNotEmpty) {
        resultString += "_";
      }
      resultString += "$name:$value";
    }
    return resultString;
  }

  void _onStoryBlockPlatformViewCreated(StoryBlockPlatformHandler handler) {
    if (_controller == null) {
      _controller = StoryBlockController(handler: handler);
      widget.onStoryBlockCreated?.call(_controller!);
    } else {
      _controller!.handler = handler;
    }
  }
}
