import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import '../../models/method_names.dart';
import '../../utils/fw_method_channel_util.dart';
import '../../utils/model_utils/fw_error_util.dart';
import '../../utils/model_utils/method_names_util.dart';
import '../../utils/model_utils/story_block_configuration_util.dart';
import 'story_block_platform_view.dart';

class StoryBlockPlatformHandler {
  final MethodChannel _channel;
  StoryBlockPlatformView _widget;

  StoryBlockPlatformHandler({
    required int viewId,
    required StoryBlockPlatformView widget,
  })  : _channel = FWMethodChannelUtil.getStoryBlockChannel(viewId),
        _widget = widget {
    _channel.setMethodCallHandler(_handleMethodCall);
  }

  void updateWidget(StoryBlockPlatformView widget) {
    if (defaultTargetPlatform == TargetPlatform.android) {
      final newStoryBlockConfigurationJson =
          StoryBlockConfigurationUtil.toJson(widget.storyBlockConfiguration);
      if (_widget.storyBlockConfiguration != widget.storyBlockConfiguration) {
        _setStoryBlockConfiguration(newStoryBlockConfigurationJson);
      }
    }
    _widget = widget;
  }

  Future<void> play() async {
    final nativeMethod = StoryBlockNativeMethodName.play.name;
    await _channel.invokeMethod(nativeMethod);
  }

  Future<void> pause() async {
    final nativeMethod = StoryBlockNativeMethodName.pause.name;
    await _channel.invokeMethod(nativeMethod);
  }

  Future<void> setMarginRight(double marginRight) async {
    if (defaultTargetPlatform == TargetPlatform.android) {
      final nativeMethod = StoryBlockNativeMethodName.setMarginRight.name;
      await _channel.invokeMethod(
        nativeMethod,
        marginRight,
      );
    }
  }

  Future<void> show() async {
    if (defaultTargetPlatform == TargetPlatform.android) {
      final nativeMethod = StoryBlockNativeMethodName.show.name;
      await _channel.invokeMethod(
        nativeMethod,
      );
    }
  }

  Future<void> hide() async {
    if (defaultTargetPlatform == TargetPlatform.android) {
      final nativeMethod = StoryBlockNativeMethodName.hide.name;
      await _channel.invokeMethod(
        nativeMethod,
      );
    }
  }

  Future<dynamic> _handleMethodCall(MethodCall call) async {
    final methodName = StoryBlockMethodNameUtil.createMethodName(call.method);
    if (methodName == null) {
      return null;
    }

    switch (methodName) {
      case StoryBlockMethodName.onStoryBlockLoadFinished:
        final error = FWErrorUtil.fromJson(call.arguments);
        _widget.onStoryBlockPlatformViewLoadFinished?.call(error);
        break;
      case StoryBlockMethodName.onStoryBlockFullScreenStateChanged:
        bool isFullScreen = false;

        if (call.arguments is Map<dynamic, dynamic>) {
          final arg = call.arguments as Map<dynamic, dynamic>;
          if (arg["isFullScreen"] is bool) {
            isFullScreen = arg["isFullScreen"] as bool;
          }
        }
        _widget.onStoryBlockPlatformViewFullScreenStateChanged
            ?.call(isFullScreen);
        break;
    }

    return null;
  }

  void _setStoryBlockConfiguration(Map<String, dynamic> playerConfigJson) {
    final nativeMethod =
        StoryBlockNativeMethodName.setStoryBlockConfiguration.name;

    _channel.invokeMethod(
      nativeMethod,
      playerConfigJson,
    );
  }
}
