import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';

import '../../models/ad_configuration.dart';
import '../../models/fw_error.dart';
import '../../models/story_block_configuration.dart';
import '../../models/story_block_source.dart';
import '../../utils/model_utils/ad_configuration_util.dart';
import '../../utils/model_utils/story_block_configuration_util.dart';
import '../mixins/android_platform_view_mixin.dart';
import 'story_block_platform_handler.dart';

typedef StoryBlockPlatformViewLoadFinishedCallback = void Function(
    FWError? error);
typedef StoryBlockPlatformViewCreatedCallback = void Function(
    StoryBlockPlatformHandler handler);
typedef StoryBlockPlatformViewFullScreenStateChanged = void Function(
    bool isFullScreen);

class StoryBlockPlatformView extends StatefulWidget {
  final StoryBlockSource source;
  final String? channel;
  final String? playlist;
  final Map<String, List<String>>? dynamicContentParameters;
  final String? hashtagFilterExpression;
  final bool? enablePictureInPicture;
  final AdConfiguration? adConfiguration;
  final StoryBlockConfiguration? storyBlockConfiguration;
  final StoryBlockPlatformViewCreatedCallback? onStoryBlockformViewCreated;
  final StoryBlockPlatformViewLoadFinishedCallback?
      onStoryBlockPlatformViewLoadFinished;
  final StoryBlockPlatformViewFullScreenStateChanged?
      onStoryBlockPlatformViewFullScreenStateChanged;
  const StoryBlockPlatformView({
    Key? key,
    required this.source,
    this.channel,
    this.playlist,
    this.dynamicContentParameters,
    this.hashtagFilterExpression,
    this.enablePictureInPicture,
    this.adConfiguration,
    this.storyBlockConfiguration,
    this.onStoryBlockformViewCreated,
    this.onStoryBlockPlatformViewLoadFinished,
    this.onStoryBlockPlatformViewFullScreenStateChanged,
  }) : super(key: key);

  @override
  State<StoryBlockPlatformView> createState() => _StoryBlockPlatformViewState();
}

class _StoryBlockPlatformViewState extends State<StoryBlockPlatformView>
    with AndroidPlatformViewMixin {
  final String _viewType = "fw.fondor.view/story_block";
  Completer<StoryBlockPlatformHandler> _handlerCompleter = Completer();

  @override
  void initState() {
    super.initState();
  }

  @override
  void didUpdateWidget(covariant StoryBlockPlatformView oldWidget) {
    super.didUpdateWidget(oldWidget);

    _handlerCompleter.future.then((handler) {
      handler.updateWidget(widget);
    });
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (defaultTargetPlatform == TargetPlatform.android) {
      return buildAndroidPlatformView(_buildAndroidPlatformView);
    } else {
      return _buildIOSPlatformView(context);
    }
  }

  Widget _buildAndroidPlatformView(BuildContext context, ValueKey key) {
    final creationParams = _generateCreationParams();
    const layoutDirection = TextDirection.ltr;
    const creationParamsCodec = StandardMessageCodec();
    return PlatformViewLink(
      key: ValueKey(key),
      viewType: _viewType,
      surfaceFactory:
          (BuildContext context, PlatformViewController controller) {
        return AndroidViewSurface(
          controller: controller as AndroidViewController,
          gestureRecognizers: _generateGestureRecognizers(),
          hitTestBehavior: PlatformViewHitTestBehavior.opaque,
        );
      },
      onCreatePlatformView: (PlatformViewCreationParams params) {
        return PlatformViewsService.initExpensiveAndroidView(
          id: params.id,
          viewType: _viewType,
          layoutDirection: layoutDirection,
          creationParams: creationParams,
          creationParamsCodec: creationParamsCodec,
          onFocus: () {
            params.onFocusChanged(true);
          },
        )
          ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
          ..addOnPlatformViewCreatedListener(_onPlatformViewCreated)
          ..create();
      },
    );
  }

  Widget _buildIOSPlatformView(BuildContext context) {
    Map<String, dynamic> creationParams = _generateCreationParams();
    const layoutDirection = TextDirection.ltr;

    return UiKitView(
      viewType: _viewType,
      onPlatformViewCreated: _onPlatformViewCreated,
      layoutDirection: layoutDirection,
      creationParams: creationParams,
      creationParamsCodec: const StandardMessageCodec(),
      gestureRecognizers: _generateGestureRecognizers(),
    );
  }

  Map<String, dynamic> _generateCreationParams() {
    Map<String, dynamic> creationParams = <String, dynamic>{
      "source": widget.source.name,
    };

    if (widget.channel != null) {
      creationParams["channel"] = widget.channel!;
    }

    if (widget.playlist != null) {
      creationParams["playlist"] = widget.playlist!;
    }

    if (widget.dynamicContentParameters != null) {
      creationParams["dynamicContentParameters"] =
          widget.dynamicContentParameters!;
    }

    if (widget.hashtagFilterExpression != null) {
      creationParams["hashtagFilterExpression"] =
          widget.hashtagFilterExpression!;
    }

    if (defaultTargetPlatform == TargetPlatform.iOS) {
      if (widget.enablePictureInPicture != null) {
        creationParams["enablePictureInPicture"] =
            widget.enablePictureInPicture!;
      }
    }

    if (widget.adConfiguration != null) {
      creationParams["adConfiguration"] =
          AdConfigurationUtil.toJson(widget.adConfiguration!);
    }

    if (defaultTargetPlatform == TargetPlatform.android) {
      if (widget.storyBlockConfiguration != null) {
        creationParams["storyBlockConfiguration"] =
            StoryBlockConfigurationUtil.toJson(widget.storyBlockConfiguration);
      }

      creationParams["marginRight"] = androidMarginRight;
    }

    return creationParams;
  }

  Set<Factory<OneSequenceGestureRecognizer>> _generateGestureRecognizers() {
    return <Factory<OneSequenceGestureRecognizer>>{
      Factory<OneSequenceGestureRecognizer>(
        () => HorizontalDragGestureRecognizer(),
      ),
    };
  }

  void _onPlatformViewCreated(int viewId) {
    final handler = StoryBlockPlatformHandler(viewId: viewId, widget: widget);
    if (_handlerCompleter.isCompleted) {
      _handlerCompleter = Completer();
    }
    _handlerCompleter.complete(handler);

    widget.onStoryBlockformViewCreated?.call(handler);
  }
}
